module HasVersions
  module Record
    extend ActiveSupport::Concern

    included do
      class_attribute :version_class_name
      self.version_class_name = '::Version'

      include HasVersions::Record::Attributes
      include HasVersions::Record::Reset
    end

    module ClassMethods
      def version_model
        @version_model ||= begin
          version_class.class_eval do
          end
          version_class
        end
      end

      def versioning_configuration
        @versioning_configuration ||= HasVersions::Record::Configuration.new
      end

      def version_class
        @version_class ||= version_class_name.constantize
      end
    end

    def versioning_configuration
      self.class.versioning_configuration
    end

    def to_version
      self.class.version_model.new(
        snapshot: versioned_attributes,
        target: self
      )
    end
  end
end
