module HasVersions
  module Record
    module Attributes
      extend ActiveSupport::Concern

      module ClassMethods
        def versioned_attributes
          versioning_configuration.attributes
        end

        def create_versioned_attributes(&block)
          versioned_attributes.inject({}) do |memo, attribute|
            if (value = block.call(attribute)).present?
              memo[attribute] = versioning_encode_value(attribute, value)
            end
            memo
          end
        end
      end

      def versioned_attributes
        self.class.create_versioned_attributes do |attribute|
          self[attribute]
        end
      end

      def versioned_attributes_was
        self.class.create_versioned_attributes do |attribute|
          send("#{attribute}_was")
        end
      end
    end
  end
end
